# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from libtaskotron.directives import xunit_directive
from libtaskotron.exceptions import TaskotronDirectiveError
from libtaskotron.check import import_YAML

import mock
import pytest
import xunitparser
import io

OUTPUT = 'Kilo kabanosu naprudko'

XML = """<?xml version="1.0"?>
<testsuite errors="0" failures="0" name="pytest" skips="0" tests="3" time="17.131">
<testcase classname="test.TestClass" file="test.py" line="30" name="test1" time="2.76759004593">
<system-out></system-out></testcase>
<testcase classname="test.TestClass" file="test.py" line="34" name="test2" time="2.67303919792">
<system-out></system-out></testcase>
<testcase classname="test.TestClass" file="test.py" line="52" name="test3" time="11.6780951023">
<system-out></system-out></testcase></testsuite>"""


class TestCheckParams(object):
    def test_file_present(self, monkeypatch):
        test_params = {'file': 'json.xml'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}

        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with mock.patch('__builtin__.open', mock.mock_open(), create=True):
            with mock.patch('libtaskotron.directives.xunit_directive.export_YAML',
                            mock.MagicMock(return_value=OUTPUT)):
                output = test_directive.process(test_params, test_arg_data)

        assert output == OUTPUT

    def test_file_not_present(self, monkeypatch):
        test_params = {'fila': 'json.xml'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with mock.patch('__builtin__.open', mock.mock_open(), create=True):
            with pytest.raises(TaskotronDirectiveError):
                test_directive.process(test_params, test_arg_data)

    def test_item_not_present(self, monkeypatch):
        test_params = {'file': 'json.xml'}
        test_arg_data = {'item': None, 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with mock.patch('__builtin__.open', mock.mock_open(), create=True):
            with pytest.raises(TaskotronDirectiveError):
                test_directive.process(test_params, test_arg_data)

    def test_type_not_present(self, monkeypatch):
        test_params = {'file': 'json.xml'}
        test_arg_data = {'item': 'starobrno', 'type': None}
        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with mock.patch('__builtin__.open', mock.mock_open(), create=True):
            with pytest.raises(TaskotronDirectiveError):
                test_directive.process(test_params, test_arg_data)

    def test_file_does_not_exist(self, monkeypatch):
        test_params = {'file': 'json.xml'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with pytest.raises(IOError):
            test_directive.process(test_params, test_arg_data)

    def test_aggregation_unknown(self, monkeypatch):
        test_params = {'file': 'json.xml', 'aggregation': 'allfail'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        stub_parse = mock.MagicMock(return_value=([], True))
        monkeypatch.setattr(xunitparser, 'parse', stub_parse)

        with pytest.raises(TaskotronDirectiveError):
            test_directive.process(test_params, test_arg_data)

    def test_aggregation_none(self, monkeypatch):
        test_params = {'file': 'json.xml', 'aggregation': 'none'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek', 'checkname': 'ochutnavka'}
        test_directive = xunit_directive.XunitDirective()

        # https://bugzilla.redhat.com/show_bug.cgi?id=1244145#c18
        # with mock.patch('__builtin__.open', mock.mock_open(read_data=XML), create=True):
        with mock.patch('__builtin__.open', mock.MagicMock(return_value=io.BytesIO(XML)),
                        create=True):
            output = test_directive.process(test_params, test_arg_data)

        assert len(import_YAML(output)) == 3

    def test_aggregation_allpass(self, monkeypatch):
        test_params = {'file': 'json.xml', 'aggregation': 'allpass'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        # ditto
        with mock.patch('__builtin__.open', mock.MagicMock(return_value=io.BytesIO(XML)),
                        create=True):
            output = test_directive.process(test_params, test_arg_data)

        assert len(import_YAML(output)) == 1

    def test_aggregation_allpass_default(self, monkeypatch):
        test_params = {'file': 'json.xml'}
        test_arg_data = {'item': 'starobrno', 'type': 'skopek'}
        test_directive = xunit_directive.XunitDirective()

        # ditto
        with mock.patch('__builtin__.open', mock.MagicMock(return_value=io.BytesIO(XML)),
                        create=True):
            output = test_directive.process(test_params, test_arg_data)

        assert len(import_YAML(output)) == 1
